#ifndef WOLF_PROCESSOR_GNSS_SINGLE_DIFF_H
#define WOLF_PROCESSOR_GNSS_SINGLE_DIFF_H

// Wolf includes
#include "core/processor/processor_base.h"
#include "gnss/capture/capture_gnss_single_diff.h"
#include "gnss/sensor/sensor_gnss.h"

// Std includes


namespace wolf {
    
WOLF_PTR_TYPEDEFS(ProcessorGnssSingleDiff);
WOLF_STRUCT_PTR_TYPEDEFS(ProcessorParamsGnssSingleDiff);


struct ProcessorParamsGnssSingleDiff : public ProcessorParamsBase
{
    Scalar time_th;
    Scalar dist_traveled;
    Scalar enu_map_init_dist_min;
    ProcessorParamsGnssSingleDiff() = default;
    ProcessorParamsGnssSingleDiff(std::string _unique_name, const ParamsServer& _server):
        ProcessorParamsBase(_unique_name, _server)
    {
        time_th                 = _server.getParam<Scalar>(_unique_name + "/time_th");
        dist_traveled           = _server.getParam<Scalar>(_unique_name + "/dist_traveled");
        enu_map_init_dist_min   = _server.getParam<Scalar>(_unique_name + "/enu_map_init_dist_min");
    }
    std::string print() const
    {
        return "\n" + ProcessorParamsBase::print()                              + "\n"
            + "time_th: "               + std::to_string(time_th)               + "\n"
            + "dist_traveled: "         + std::to_string(dist_traveled)         + "\n"
            + "enu_map_init_dist_min: " + std::to_string(enu_map_init_dist_min) + "\n";
    }
};
    
//class
class ProcessorGnssSingleDiff : public ProcessorBase
{
    protected:
        SensorGnssPtr sensor_gnss_;
        ProcessorParamsGnssSingleDiffPtr params_gnss_;
        CaptureGnssSingleDiffPtr incoming_capture_;
        FrameBasePtr last_KF_;

    public:
        ProcessorGnssSingleDiff(ProcessorParamsGnssSingleDiffPtr _params_gnss);
        virtual ~ProcessorGnssSingleDiff();
        virtual void configure(SensorBasePtr _sensor) override;
        FrameBasePtr getLastKF();

    protected:
        /** \brief process an incoming capture
         *
         * Each derived processor should implement this function. It will be called if:
         *  - A new capture arrived and triggerInCapture() returned true.
         */
        virtual void processCapture(CaptureBasePtr) override;

        /** \brief process an incoming key-frame
         *
         * The ProcessorTracker only processes incoming captures (it is not called).
         */
        virtual void processKeyFrame(FrameBasePtr _keyframe, const Scalar& _time_tolerance) override {};

        /** \brief store key frame
        *
        * Returns true if the key frame should be stored
        */
        virtual bool storeKeyFrame(FrameBasePtr);

        /** \brief store capture
        *
        * Returns true if the capture should be stored
        */
        virtual bool storeCapture(CaptureBasePtr);

        /** \brief trigger in capture
         *
         * Returns true if processCapture() should be called after the provided capture arrived.
         */
        virtual bool triggerInCapture(CaptureBasePtr) const override;

        /** \brief trigger in key-frame
         *
         * The ProcessorTracker only processes incoming captures, then it returns false.
         */
        virtual bool triggerInKeyFrame(FrameBasePtr _keyframe, const Scalar& _time_tolerance) const override {return false;}

        virtual bool voteForKeyFrame() const override;


    private:
        FactorBasePtr emplaceFactor(FeatureBasePtr _ftr);

    public:
        static ProcessorBasePtr create(const std::string& _unique_name, const ProcessorParamsBasePtr _params);

};

inline bool ProcessorGnssSingleDiff::triggerInCapture(CaptureBasePtr) const
{
    return true;
}

inline wolf::FrameBasePtr ProcessorGnssSingleDiff::getLastKF()
{
    return last_KF_;
}

} // namespace wolf

#endif //WOLF_PROCESSOR_GNSS_SINGLE_DIFF_H
